﻿Public MustInherit Class Brush : Implements Api.Brush
    Protected _image As System.Drawing.Image
    Protected _separation As Single
    Protected __p As Api.StrokePoint

    Public Overridable Property Separation() As Single Implements Api.Brush.Separation
        Get
            Separation = _separation
        End Get
        Set(ByVal value As Single)
            _separation = value
        End Set
    End Property

    Public Overridable Property Image(ByVal point As Api.StrokePoint) As System.Drawing.Image Implements Api.Brush.Image
        Get
            If point.Pressure > 0 Then
                Image = Me._image
            Else
                Image = New Drawing.Bitmap(3, 3)
                Dim g = Drawing.Graphics.FromImage(Image)
                g.FillRectangle(Drawing.Brushes.Pink, Image.GetBounds(Drawing.GraphicsUnit.Pixel))
                g.Dispose()
            End If
        End Get
        Set(ByVal value As System.Drawing.Image)
            Me._image = value
        End Set
    End Property


    Public Overridable Sub Draw(ByVal canvas As Api.Canvas, ByVal points As List(Of Api.StrokePoint)) Implements Api.Brush.Draw
        If points.Count > 0 Then
            For Each p In points
                If Me.__p Is Nothing Then
                    Me.__p = p
                ElseIf Math.Sqrt((p.Location.X - Me.__p.Location.X) ^ 2 + (p.Location.Y - Me.__p.Location.Y) ^ 2) >= Me._separation Then
                    With Me
                        Dim location = p.Location
                        Dim i = Me.Image(p)
                        If i IsNot Nothing Then
                            canvas.Draw(i, Drawing.Point.Truncate(Drawing.PointF.Add(location, New Drawing.SizeF(-i.Width / 2, -i.Height / 2))))
                        End If
                    End With
                    Me.__p = p
                End If
            Next
        End If
    End Sub

    Public Sub Initialize() Implements Api.Brush.Initialize
        Me._image = New System.Drawing.Bitmap(5, 5)
        Me._separation = (Me._image.Size.Width + Me._image.Size.Height) * 5
        Me.__p = New Plugin.StrokePoint(System.Drawing.Point.Empty, 0)

        Dim g = System.Drawing.Graphics.FromImage(Me._image)
        g.FillRectangle(Drawing.Brushes.Red, Me._image.GetBounds(System.Drawing.GraphicsUnit.Pixel))
        g.Dispose()
    End Sub

    Public Sub New()
        MyClass.Initialize()
    End Sub

End Class
